from os import path
import json

def clean(d):
    if isinstance(d, dict):
        d1 = {}
        for k in d.keys():
            v = d[k]
            if isinstance(v, dict):
                v = clean(v)
            elif isinstance(v, int):
                v = 0
            elif isinstance(v, float):
                v = 0.0
            elif isinstance(v, list):
                v = clean(v)
            else:
                v = 'xxx'
            d1[k] = v
        return d1
    elif isinstance(d, list):
        d1 = []
        for v in d:
            d1.append(clean(v))
        return d1
    return d

def dict_read(filename):
    basepath = path.dirname(__file__)
    filepath = path.join(basepath, filename)
    json_str = open(filepath, 'r').read()
    return json.loads(json_str)

def dict_compare_keys(d1, d2, key_path=''):
    ''' Compare two dicts recursively and see if dict1 has any keys that dict2 does not
    Returns: list of key paths
    '''
    res = []
    if not d1:
        return res
    if not isinstance(d1, dict):
        return res
    for k in d1:
        if k not in d2:
            missing_key_path = f'{key_path}->{k}'
            res.append(missing_key_path)
        else:
            if isinstance(d1[k], dict):
                key_path1 = f'{key_path}->{k}'
                res1 = dict_compare_keys(d1[k], d2[k], key_path1)
                res = res + res1
            elif isinstance(d1[k], list):
                key_path1 = f'{key_path}->{k}[0]'
                dv1 = d1[k][0] if len(d1[k]) > 0 else None
                dv2 = d2[k][0] if len(d2[k]) > 0 else None
                res1 = dict_compare_keys(dv1, dv2, key_path1)
                res = res + res1
    return res


import logging
import re
import pytest
from .common import dict_read, dict_compare_keys

logger = logging.getLogger(__name__)

@pytest.fixture
def contacts_view_id(fs):
    views = fs.contacts.get_views()
    for v in views:
        if re.match('all', v['name'].lower()):
            return v['id']
    assert False, 'Could not find a contacts view'

def assert_contact_well_formed(contact):
    ref_contact = dict_read('contact.json')
    logger.debug('contact = %s', contact)
    logger.debug('ref_contact = %s', ref_contact)
    diff = dict_compare_keys(contact, ref_contact)
    logger.debug('dict_compare = %s', diff)
    assert diff == [], 'unexpected contact structure'

def test_contacts_get_views(fs):
    views = fs.contacts.get_views()
    assert views

def test_contacts_get_all_generator(fs, contacts_view_id):
    for contact in fs.contacts.get_all_generator(view_id=contacts_view_id, limit=10):
        assert_contact_well_formed(contact)

def test_contacts_get(fs, contacts_view_id):
    contact1 = next(fs.contacts.get_all_generator(view_id=contacts_view_id, limit=1))
    assert_contact_well_formed(contact1)
    contact2 = fs.contacts.get(id=contact1['id'])
    assert_contact_well_formed(contact2)



import logging
import re
import pytest
from .common import dict_read, dict_compare_keys

logger = logging.getLogger(__name__)

@pytest.fixture
def deals_view_id(fs):
    views = fs.deals.get_views()
    for v in views:
        if re.match('all', v['name'].lower()):
            return v['id']
    assert False, 'Could not find a deals view'

def assert_deal_well_formed(deal):
    ref_deal = dict_read('deal.json')
    logger.debug('deal = %s', deal)
    logger.debug('ref_deal = %s', ref_deal)
    diff = dict_compare_keys(deal, ref_deal)
    logger.debug('dict_compare = %s', diff)
    assert diff == [], 'unexpected deal structure'

def test_deals_get_views(fs):
    views = fs.deals.get_views()
    assert views

def test_deals_get_all_generator(fs, deals_view_id):
    for deal in fs.deals.get_all_generator(view_id=deals_view_id, limit=10):
        assert_deal_well_formed(deal)

def test_deals_get(fs, deals_view_id):
    deal = next(fs.deals.get_all_generator(view_id=deals_view_id, limit=1))
    assert_deal_well_formed(deal)



import logging
import re
import pytest
from .common import dict_read, dict_compare_keys

logger = logging.getLogger(__name__)

@pytest.fixture
def accounts_view_id(fs):
    views = fs.accounts.get_views()
    for v in views:
        if re.match('all', v['name'].lower()):
            return v['id']
    assert False, 'Could not find a accounts view'

def assert_account_well_formed(account):
    ref_account = dict_read('account.json')
    logger.debug('account = %s', account)
    logger.debug('ref_account = %s', ref_account)
    diff = dict_compare_keys(account, ref_account)
    logger.debug('dict_compare = %s', diff)
    assert diff == [], 'unexpected account structure'

def test_accounts_get_views(fs):
    views = fs.accounts.get_views()
    assert views

def test_accounts_get_all_generator(fs, accounts_view_id):
    for account in fs.accounts.get_all_generator(view_id=accounts_view_id, limit=10):
        assert_account_well_formed(account)

def test_accounts_get(fs, accounts_view_id):
    account = next(fs.accounts.get_all_generator(view_id=accounts_view_id, limit=1))
    assert_account_well_formed(account)
